@@ -1,6 +1,6 @@
 import copy
 import math
-from functools import partial
+from functools import partial, wraps
 from math import sqrt
 
 from vector_quantize_pytorch import VectorQuantize as VQ
@@ -25,6 +25,8 @@ def exists(val):
 def default(val, d):
     return val if exists(val) else d
 
+# decorators
+
 def eval_decorator(fn):
     def inner(model, *args, **kwargs):
         was_training = model.training
@@ -34,6 +36,22 @@ def eval_decorator(fn):
         return out
     return inner
 
+def remove_vgg(fn):
+    @wraps(fn)
+    def inner(self, *args, **kwargs):
+        has_vgg = hasattr(self, 'vgg')
+        if has_vgg:
+            vgg = self.vgg
+            delattr(self, 'vgg')
+
+        out = fn(self, *args, **kwargs)
+
+        if has_vgg:
+            self.vgg = vgg
+
+        return out
+    return inner
+
 # keyword argument helpers
 
 def pick_and_pop(keys, d):
@@ -302,6 +320,7 @@ class VQGanVAE(nn.Module):
         self.image_size = image_size
         self.channels = channels
         self.num_layers = num_layers
+        self.fmap_size = image_size // (num_layers ** 2)
         self.codebook_size = vq_codebook_size
 
         self.encoders = MList([])
@@ -394,29 +413,13 @@ class VQGanVAE(nn.Module):
 
         return vae_copy.to(device)
 
+    @remove_vgg
     def state_dict(self, *args, **kwargs):
-        # make sure VGG is not saved in state dictionary
-        has_vgg = hasattr(self, 'vgg')
-        if has_vgg:
-            vgg = self.vgg
-            delattr(self, 'vgg')
-
-        state_dict = super().state_dict(*args, **kwargs)
-
-        if has_vgg:
-            self.vgg = vgg
-        return state_dict
+        return super().state_dict(*args, **kwargs)
 
+    @remove_vgg
     def load_state_dict(self, *args, **kwargs):
-        has_vgg = hasattr(self, 'vgg')
-        if has_vgg:
-            vgg = self.vgg
-            delattr(self, 'vgg')
-
-        super().load_state_dict(*args, **kwargs)
-
-        if has_vgg:
-            self.vgg = vgg
+        return super().load_state_dict(*args, **kwargs)
 
     @property
     def codebook(self):
@@ -434,6 +437,15 @@ class VQGanVAE(nn.Module):
 
         return fmap
 
+    @torch.no_grad()
+    @eval_decorator
+    def codebook_indices_to_video(self, indices):
+        b = indices.shape[0]
+        codes = self.codebook[indices]
+        codes = rearrange(codes, 'b (f h w) d -> (b f) d h w', h = self.fmap_size, w = self.fmap_size)
+        video = self.decode(codes)
+        return rearrange(video, '(b f) ... -> b f ...', b = b)
+
     @torch.no_grad()
     @eval_decorator
     def get_video_indices(self, video):
